@@ -5,9 +5,9 @@
 from __future__ import print_function, unicode_literals
 
 import conditions
-import mozdevice
 import mozfile
 import os
+import usb
 
 from mach.decorators import (
     CommandArgument,
@@ -20,21 +20,6 @@ from mozbuild.base import (
 )
 from mozprocess import ProcessHandler
 
-DEVICE_NOT_FOUND = '''
-The %s command cannot find a device.
-
-Please make sure your device is connected.
-'''.lstrip()
-
-def _is_device_attached(self):
-    """Returns True if a device is attached, False if not."""
-    try:
-        dm = mozdevice.DeviceManagerADB()
-        dm.devices()
-    except mozdevice.DMError:
-        print(DEVICE_NOT_FOUND % 'flash')
-        return False
-    return True
 
 @CommandProvider
 class Build(MachCommandBase):
@@ -46,7 +31,7 @@ class Build(MachCommandBase):
         self.device_name = context.device_name
 
     @Command('clobber', category='build',
-        conditions=[build_conditions.is_b2g],
+        conditions=[lambda x: True],
         description='Clobber the tree (delete the objdir and out directory).')
     def clobber(self):
         self.remove_objdir()
@@ -60,6 +45,8 @@ class Build(MachCommandBase):
         description='Run the build script.')
     @CommandArgument('modules', nargs='*',
                      help='Only build specified sub-modules')
+    @CommandArgument('-j', action='store', dest='cores',
+                     help='Number of cores to use')
     @CommandArgument('--debug', action='store_true',
                      help='Enable a debug build')
     @CommandArgument('--profiling', action='store_true',
@@ -70,32 +57,36 @@ class Build(MachCommandBase):
                      help='Disable optimizer')
     @CommandArgument('--valgrind', action='store_true',
                      help='Enable valgrind')
-    def build_script(self, modules=None, debug=False, profiling=False,
-                     noftu=False, noopt=False, valgrind=False):
+    def build_script(self, modules=None, cores=None, debug=False,
+                     profiling=False, noftu=False, noopt=False, valgrind=False):
         command = [os.path.join(self.b2g_home, 'build.sh')]
+        env = os.environ.copy()
 
         if modules:
             command.extend(modules)
 
+        if cores:
+            command.append('-j%s' % cores)
+
         if debug:
-            command.insert(0, 'B2G_DEBUG=1')
+            env['B2G_DEBUG'] = '1'
 
         if profiling:
-            command.insert(0, 'MOZ_PROFILING=1')
+            env['MOZ_PROFILING'] = '1'
 
         if noftu:
-            command.insert(0, 'NOFTU=1')
+            env['NOFTU'] = '1'
 
         if noopt:
             if profiling:
                 print("Can't perform profiling if optimizer is disabled")
                 return 1
-            command.insert(0, 'B2G_NOOPT=1')
+            env['B2G_NOOPT'] = '1'
 
         if valgrind:
-            command.insert(0, 'B2G_VALGRIND=1')
+            env['B2G_VALGRIND'] = '1'
 
-        p = ProcessHandler(command)
+        p = ProcessHandler(command, env=env)
         p.run()
 
         #TODO: Error checking.
@@ -105,10 +96,20 @@ class Build(MachCommandBase):
         conditions=[build_conditions.is_b2g,
                     conditions.is_device],
         description='Flash the current B2G build onto a device.')
-    def flash(self):
+    @CommandArgument('partitions', nargs='*',
+                     help='Only flash specified partitions')
+    @CommandArgument('--app', action='store',
+                     help='Update only a specific app')
+    def flash(self, partitions=None, app=None):
+        usb.verify_device('flash')
+
         command = os.path.join(self.b2g_home, 'flash.sh')
+        if partitions:
+            command.extend(partitions)
+
+        if app:
+            command.insert(0, 'BUILD_APP_NAME=%s' % app)
+
         p = ProcessHandler(command)
-        if _is_device_attached():
-            p.run()
-            return p.wait()
-        return 1
+        p.run()
+        return p.wait()
